#include "mex.h"
#include "stdint.h"
#include "libLiveTrack.h"

#define VET_ASSERT(TEST,ERRORMESSAGE) ( (TEST) ? (void)0 : mexErrMsgTxt(ERRORMESSAGE) )

/* *****************************************************************************/
#define GET_pOUT(pDOUBLE_OUT,pMXARRAY_OUT,nROWS,nCOLS)  VET_ASSERT(0<nROWS,"Error in crsLiveTrack: Cannot create a matrix smaller than 1-by-1 in size"); \
                                                        VET_ASSERT(0<nCOLS,"Error in crsLiveTrack: Cannot create a matrix smaller than 1-by-1 in size"); \
                                                        pMXARRAY_OUT = mxCreateDoubleMatrix(nROWS,nCOLS,mxREAL);                                   \
                                                        VET_ASSERT(pMXARRAY_OUT!=NULL,"Out of memory - not enough free heap space");               \
                                                        pDOUBLE_OUT = (double *)mxGetPr(pMXARRAY_OUT);                                             \
                                                        VET_ASSERT(pDOUBLE_OUT!=NULL,"No real data in mxarray")

/* ****************************************************************************

* written by Mark Hodgetts and Jakob Thomassen
* usage on Windows:
* mex crsLiveTrack.c libLiveTrack.lib  
* usage on Linux:
* mex crsLiveTrack.c /usr/lib/libLiveTrack.so


* ****************** The subfunctions *********************  */
void mexInit(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexInit:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t)crsLiveTrackInit();
}

void mexClose(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
   mexErrMsgIdAndTxt( "MATLAB:mexClose:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t)crsLiveTrackClose();
}

void mexSetResultsType(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	int type;
	
	if(nrhs!=2) 
   mexErrMsgIdAndTxt( "MATLAB:mexSetResultsType:invalidNumInputs",
            "Two inputs required.");
	
	type = (int) mxGetScalar(prhs[1]);
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackSetResultsType(type);
}

void mexStartTracking(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexStartTracking:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackStartTracking();
}

void mexStopTracking(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexStopTracking:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t)crsLiveTrackStopTracking();
}

void mexGetResultsCount(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexGetResultsCount:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackGetResultsCount();
}

void mexGetResults(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
   T_RESULTS_STRUCT result;
	int32_t err;
    bool removeFromBuffer; 
	int i, numResults, firstOrLast;
	
	double              *Timestamp;
	double              *ResultsType;
	
	double              *LeftPupilX;
	double              *LeftPupilY;
	double              *LeftGlintX;
	double              *LeftGlintY;
	double              *LeftPupilMajorAxis;
	double              *LeftPupilMinorAxis;
	double              *LeftGazeX;
	double              *LeftGazeY;
	double              *LeftGazeZ;
	double              *LeftGazeAzimuth;
	double              *LeftGazeElevation;
	double              *LeftGazeLongitude;
	double              *LeftGazeLatitude;	
	double              *LeftTracked; 
    double              *LeftEnabled; 	  
    double              *LeftCalibrated;
    double              *LeftDropped;
    double              *LeftROI;
	
	double              *RightPupilX;
	double              *RightPupilY;
	double              *RightGlintX;
	double              *RightGlintY;
	double              *RightPupilMajorAxis;
	double              *RightPupilMinorAxis;
	double              *RightGazeX;
	double              *RightGazeY;
	double              *RightGazeZ;
	double              *RightGazeAzimuth;
	double              *RightGazeElevation;
	double              *RightGazeLongitude;
	double              *RightGazeLatitude;	
	double              *RightTracked; 
	double              *RightEnabled; 	  
    double              *RightCalibrated;
    double              *RightDropped;
    double              *RightROI;
	
	double              *DigitalIO;
    double              *FixationDetected;
    
	if(nrhs!=4) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetBufferedResults:invalidNumInputs",
            "Four inputs required!"); 
				
    numResults = (int) mxGetScalar(prhs[1]);
    removeFromBuffer = (bool) mxGetScalar(prhs[2]);
    firstOrLast = (int) mxGetScalar(prhs[3]);
    
    if(numResults==-1){
        numResults = crsLiveTrackGetResultsCount();
    }
    
    if(firstOrLast){
        crsLiveTrackSetBufferPosition(0);
    }
    else{
        crsLiveTrackSetBufferPosition(crsLiveTrackGetResultsCount()-numResults);
    }
    
	GET_pOUT(Timestamp,         	plhs[0],numResults,1);  
    GET_pOUT(ResultsType,        	plhs[1],numResults,1);
    GET_pOUT(LeftPupilX,         	plhs[2],numResults,1);  
    GET_pOUT(LeftPupilY,         	plhs[3],numResults,1);
    GET_pOUT(LeftGlintX,  			plhs[4],numResults,1);
    GET_pOUT(LeftGlintY,     		plhs[5],numResults,1);
    GET_pOUT(LeftPupilMajorAxis,  	plhs[6],numResults,1);
    GET_pOUT(LeftPupilMinorAxis,	plhs[7],numResults,1);
    GET_pOUT(LeftGazeX,				plhs[8],numResults,1);
    GET_pOUT(LeftGazeY,  			plhs[9],numResults,1);
	GET_pOUT(LeftGazeZ,  			plhs[10],numResults,1);
    GET_pOUT(LeftGazeAzimuth,  		plhs[11],numResults,1);
	GET_pOUT(LeftGazeElevation,		plhs[12],numResults,1);
    GET_pOUT(LeftGazeLongitude,     plhs[13],numResults,1);
	GET_pOUT(LeftGazeLatitude,		plhs[14],numResults,1);
	GET_pOUT(LeftTracked,  	     	plhs[15],numResults,1);
    GET_pOUT(LeftEnabled,  	     	plhs[16],numResults,1);
    GET_pOUT(LeftCalibrated,  	  	plhs[17],numResults,1);
    GET_pOUT(LeftDropped,           plhs[18],numResults,1);
    GET_pOUT(LeftROI,               plhs[19],numResults,1);
	
	GET_pOUT(RightPupilX,           plhs[20],numResults,1);
    GET_pOUT(RightPupilY,           plhs[21],numResults,1);
    GET_pOUT(RightGlintX,  			plhs[22],numResults,1);
    GET_pOUT(RightGlintY,     		plhs[23],numResults,1);
    GET_pOUT(RightPupilMajorAxis,	plhs[24],numResults,1);
    GET_pOUT(RightPupilMinorAxis,   plhs[25],numResults,1);
    GET_pOUT(RightGazeX,			plhs[26],numResults,1);
    GET_pOUT(RightGazeY,  			plhs[27],numResults,1);
	GET_pOUT(RightGazeZ,  			plhs[28],numResults,1);
    GET_pOUT(RightGazeAzimuth,  	plhs[29],numResults,1);
	GET_pOUT(RightGazeElevation,	plhs[30],numResults,1);
    GET_pOUT(RightGazeLongitude,    plhs[31],numResults,1);
	GET_pOUT(RightGazeLatitude,     plhs[32],numResults,1);
	GET_pOUT(RightTracked,          plhs[33],numResults,1); 
	GET_pOUT(RightEnabled,  	   	plhs[34],numResults,1);
    GET_pOUT(RightCalibrated,  	  	plhs[35],numResults,1);
    GET_pOUT(RightDropped,          plhs[36],numResults,1);
    GET_pOUT(RightROI,              plhs[37],numResults,1);
    
    GET_pOUT(DigitalIO,             plhs[38],numResults,1);
    GET_pOUT(FixationDetected,      plhs[39],numResults,1);
	GET_pOUT(err,      				plhs[40],numResults,1);
	
	
	for (i = 0; i < numResults; i++) {
		err = (int32_t)crsLiveTrackGetBufferedResult(&result, removeFromBuffer);
		
		if (err >= 0) 
		{
			Timestamp[i] =              (double) result.Timestamp;
			ResultsType[i] =			(double) result.ResultsType;	
			
			LeftPupilX[i] = 			(double) result.Eye[0].PupilX;
			LeftPupilY[i] = 			(double) result.Eye[0].PupilY;
			LeftGlintX[i] = 			(double) result.Eye[0].GlintX;
			LeftGlintY[i] = 			(double) result.Eye[0].GlintY;
			LeftPupilMajorAxis[i] = 	(double) result.Eye[0].PupilMajorAxis;
			LeftPupilMinorAxis[i] = 	(double) result.Eye[0].PupilMinorAxis;
			LeftGazeX[i] = 				(double) result.Eye[0].GazeX;
			LeftGazeY[i] = 				(double) result.Eye[0].GazeY;
			LeftGazeZ[i] = 				(double) result.Eye[0].GazeZ;
            LeftGazeAzimuth[i] =        (double) result.Eye[0].GazeAzimuth;
			LeftGazeElevation[i] =   	(double) result.Eye[0].GazeElevation;
            LeftGazeLongitude[i] =     	(double) result.Eye[0].GazeLongitude;
			LeftGazeLatitude[i] =      	(double) result.Eye[0].GazeLatitude;
			LeftTracked[i] =           	(double) result.Eye[0].Tracked; 
            LeftEnabled[i] =           	(double) result.Eye[0].Enabled; 
            LeftCalibrated[i] =      	(double) result.Eye[0].Calibrated;
            LeftDropped[i] =            (double) result.Eye[0].Dropped;
            LeftROI[i] =              	(double) result.Eye[0].ROI; 
		
			RightPupilX[i] = 			(double) result.Eye[1].PupilX;
			RightPupilY[i] = 			(double) result.Eye[1].PupilY;
			RightGlintX[i] = 			(double) result.Eye[1].GlintX;
			RightGlintY[i] = 			(double) result.Eye[1].GlintY;
			RightPupilMajorAxis[i] =    (double) result.Eye[1].PupilMajorAxis;
			RightPupilMinorAxis[i] =    (double) result.Eye[1].PupilMinorAxis;
			RightGazeX[i] = 			(double) result.Eye[1].GazeX;
			RightGazeY[i] = 			(double) result.Eye[1].GazeY;
			RightGazeZ[i] = 			(double) result.Eye[1].GazeZ;
			RightGazeAzimuth[i] =       (double) result.Eye[1].GazeAzimuth;
            RightGazeElevation[i] =     (double) result.Eye[1].GazeElevation;
            RightGazeLongitude[i] =     (double) result.Eye[1].GazeLongitude;
			RightGazeLatitude[i] =      (double) result.Eye[1].GazeLatitude;
			RightTracked[i] =           (double) result.Eye[1].Tracked; 
            RightEnabled[i] =           (double) result.Eye[1].Enabled; 
            RightCalibrated[i] =     	(double) result.Eye[1].Calibrated;
            RightDropped[i] =          	(double) result.Eye[1].Dropped;
            RightROI[i] =             	(double) result.Eye[1].ROI;      
            
            DigitalIO[i] =             	(double) result.DigitalIO; 
            FixationDetected[i] =     	(double) result.FixationDetected; 
		}
	} 
}

void mexClearDataBuffer(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
   mexErrMsgIdAndTxt( "MATLAB:mexClearDataBuffer:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackClearDataBuffer();
}

void mexGetLibVersion(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetLibVersion:invalidNumInputs",
            "One input required.");
	
    plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
     
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackGetLibVersion();
}

void mexCalibrateDevice(int nlhs, mxArray *plhs[],
										int nrhs, const mxArray *prhs[])
{
	double *y, rpc, viewDist, glintMedianX, glintMedianY;
	int32_t calErr;
    int i, numberOfTargets, eye;
	targ_struct *target;

	if(nrhs!=6) 
    mexErrMsgIdAndTxt( "MATLAB:mexCalibrateDevice:invalidNumInputs",
            "Six inputs required.");

    eye = (int) mxGetScalar(prhs[1]); /* 0 for left eye and 1 for right eye */
	numberOfTargets = (int) mxGetScalar(prhs[2]);
	target = (targ_struct *)calloc(numberOfTargets, sizeof(targ_struct));

	y = mxGetPr(prhs[3]);

    for (i = 0; i < numberOfTargets; i++) 
	{
        target[i].Target_X = y[i * 4];
		target[i].Target_Y = y[i * 4 + 1];
		target[i].Vector_X = y[i * 4 + 2];
		target[i].Vector_Y = y[i * 4 + 3];
    }

    rpc = 100;
	viewDist = (double) mxGetScalar(prhs[4]);

    y = mxGetPr(prhs[5]); /* Assign a pointer to the input (glintMedian) */
    glintMedianX = y[0];
    glintMedianY = y[1];

	plhs[0] = mxCreateDoubleMatrix(2, 1, mxREAL);
	/* Assign pointer to the output  */
	
	y = mxGetPr(plhs[0]);

	y[0] = (double) crsLiveTrackCalibrateDevice(eye, numberOfTargets, target, rpc, viewDist, glintMedianX, glintMedianY, &calErr);
	y[1] = (double) calErr;
	
	free(target);
} 

void mexSetCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	double rpc, cal[16], viewDist, glintMedianX, glintMedianY;
	double *y;
    int i, eye;
	
	if(nrhs!=4) 
    mexErrMsgIdAndTxt( "MATLAB:mexSetCalibration:invalidNumInputs",
            "Four inputs required.");
	eye = (int) mxGetScalar(prhs[1]); /* 0 for left eye and 1 for right eye */
    y = mxGetPr(prhs[2]); /* Assign a pointer to the input (calibration matrix) */
       
    for (i = 0; i < 16; i++) 
	{
		cal[i] = y[i];
	} 
    glintMedianX = y[16];
    glintMedianY = y[17];

	rpc=100;
    
	viewDist = (double) mxGetScalar(prhs[3]);
	
    plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
    y[0] = (int32_t) crsLiveTrackSetCalibration(eye, cal, rpc, viewDist, glintMedianX, glintMedianY);
       
}

void mexGetCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	double *y, *yy, rpc, cal[16], viewDist, glintMedianX, glintMedianY;
	int i, eye;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetCalibration:invalidNumInputs",
            "Two inputs required.");
	eye = (int) mxGetScalar(prhs[1]); /* 0 for left eye and 1 for right eye */
    
	plhs[0] = mxCreateDoubleMatrix(18, 1, mxREAL);
    plhs[1] = mxCreateDoubleMatrix(1, 1, mxREAL);
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
    crsLiveTrackGetCalibration(eye, cal, &rpc, &viewDist, &glintMedianX, &glintMedianY);
	for (i = 0; i < 16; i++) 
	{
		y[i]=(double) cal[i];
	} 
    y[16]=(double) glintMedianX;
	y[17]=(double) glintMedianY; 
    
    yy = mxGetPr(plhs[1]);
	yy[0] = (double) viewDist;
   
}

void mexGetPupilCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    double *y, pixelsToMM;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetPupilCalibration:invalidNumInputs",
            "One input required.");
	
  	plhs[0] = mxCreateDoubleMatrix(1, 1, mxREAL);
	
	crsLiveTrackGetPupilCalibration(&pixelsToMM);
    
    /* Assign pointer to the output  */
	y = mxGetPr(plhs[0]);
	y[0] = (double) pixelsToMM;
}

void mexSetPupilCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t *y;
    double diameter, pixels;
    
    if(nrhs!=3) 
    mexErrMsgIdAndTxt( "MATLAB:mexSetPupilCalibration:invalidNumInputs",
            "Three inputs required.");
	
	diameter = mxGetScalar(prhs[1]);
    pixels = mxGetScalar(prhs[2]);
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
    y[0] = (int32_t) crsLiveTrackSetPupilCalibration(diameter, pixels);
}



void mexSetTracking(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{    
    int32_t *y;
	bool leftEye, rightEye;
	
	if(nrhs!=3) 
    mexErrMsgIdAndTxt( "MATLAB:mexSetTracking:invalidNumInputs",
            "Three inputs required.");
	
	leftEye = (bool) mxGetScalar(prhs[1]);
    rightEye = (bool) mxGetScalar(prhs[2]);
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackSetTracking(leftEye, rightEye);
}

void mexGetTracking(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t *y, *y2, *y3;
	bool leftEye, rightEye;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetTracking:invalidNumInputs",
            "One input required.");

  	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[2] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
	
    /* Assign pointer to the output  */
	y = mxGetPr(plhs[0]);
    y2 = mxGetPr(plhs[1]);
    y3 = mxGetPr(plhs[2]);
    
	y3[0] = (int32_t) crsLiveTrackGetTracking(&leftEye, &rightEye);
    y[0] = (int32_t) leftEye;
	y2[0] = (int32_t) rightEye;
    
}

void mexGetSerialNumber(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    char *output_buf;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetSerialNumber:invalidNumInputs",
            "One input required.");
      
    output_buf=mxCalloc(16, sizeof(char));
     
    crsLiveTrackGetSerialNumber(output_buf, 16);   
    
     /* set C-style string output_buf to MATLAB mexFunction output*/
    plhs[0] = mxCreateString(output_buf);
}

void mexGetFirmwareVersion(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    uint16_t version, *y;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetFirmwareVersion:invalidNumInputs",
            "One input required.");
   
	plhs[0] = mxCreateNumericMatrix(1, 1, mxUINT16_CLASS, mxREAL);;
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
    crsLiveTrackGetFirmwareVersion(&version);
    y[0]= version;
}

void mexGetFirmwareBuildDateTime(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    char *output_buf;
	
	if(nrhs!=1) 
    mexErrMsgIdAndTxt( "MATLAB:mexGetFirmwareBuildDateTime:invalidNumInputs",
            "One input required.");
      
    output_buf=mxCalloc(32, sizeof(char));
     
    crsLiveTrackGetFirmwareBuildDateTime(output_buf, 32);   
    
     /* set C-style string output_buf to MATLAB mexFunction output*/
    plhs[0] = mxCreateString(output_buf);
    
}

void mexSaveCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{    
    int32_t *y;
	char *filename;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexSaveCalibration:invalidNumInputs",
            "Two inputs required.");
    
	filename = (char *) mxCalloc(mxGetN(prhs[1])+1, sizeof(char));
    mxGetString(prhs[1], filename, mxGetN(prhs[1])+1);      
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackSaveCalibration(filename);
}

void mexLoadCalibration(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{    
    int32_t *y;
	char *filename;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexLoadCalibration:invalidNumInputs",
            "Two inputs required.");
    
	filename = (char *) mxCalloc(mxGetN(prhs[1])+1, sizeof(char));
    mxGetString(prhs[1], filename, mxGetN(prhs[1])+1);      
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackLoadCalibration(filename);
}

/* Returns the device calibration state.
               Returns 0 on success. */
void mexGetCalibrated(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t *y, *y2;
    int eye;
	bool calibrated;
    
	if(nrhs!=2) 
	mexErrMsgIdAndTxt( "MATLAB:mexGetCalibrated:invalidNumInputs",
            "Two inputs required.");
	
    eye = (int) mxGetScalar(prhs[1]); /* 0 for left eye and 1 for right eye */
    
    plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y2 = mxGetPr(plhs[1]);

    y2[0] = (int32_t) crsLiveTrackGetCalibrated(eye, &calibrated);
	y[0] = (int32_t) calibrated;
}

/* Returns the device calibration state of the pupil calibration.
               Returns 0 on success. */
void mexGetPupilCalibrated(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t *y, *y2;
	bool calibrated;
    
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexGetPupilCalibrated:invalidNumInputs",
            "One input required.");
	
   
    plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y2 = mxGetPr(plhs[1]);

    y2[0] = (int32_t) crsLiveTrackGetPupilCalibrated(&calibrated);
	y[0] = (int32_t) calibrated;
}

        
        
/* Returns true if tracking data is being saved into the buffer. */
void mexGetTrackingStarted(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{    
    int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexGetTrackingStarted:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackGetTrackingStarted();
}
     

/* Gets the configured capture parameters. 
	 * Returns 0 on success.
	 * Note: rate: Configured capture rate in frames per second. 
	 * All other units are in pixels. */
void mexGetCaptureConfig(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t width, height, rate, offsetX, offsetY, ErrorCode;
    int32_t *y1, *y2, *y3, *y4, *y5, *y6;
    
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexGetCaptureConfig:invalidNumInputs",
            "One input required.");
	
    plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[1] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[2] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[3] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[4] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    plhs[5] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
    
	/* Assign pointer to the output */
	y1 = mxGetPr(plhs[0]);
	y2 = mxGetPr(plhs[1]);
    y3 = mxGetPr(plhs[2]);
	y4 = mxGetPr(plhs[3]);
    y5 = mxGetPr(plhs[4]);
    y6 = mxGetPr(plhs[5]);

    y6[0] = (int32_t) crsLiveTrackGetCaptureConfig(&width, &height, &rate, &offsetX, &offsetY);
    y1[0] = (int32_t) width;
    y2[0] = (int32_t) height;
    y3[0] = (int32_t) rate;
    y4[0] = (int32_t) offsetX;
    y5[0] = (int32_t) offsetY;

}

/* % EXPORT int32_t crsLiveTrackGetCaptureConfig(uint32_t *width, uint32_t *height, uint32_t *rate, uint32_t *offsetX, uint32_t *offsetY);

/*Starts % recording eyetracking data to a CSV file.*/
void mexSetDataFilename(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
    int32_t *y;
	char *filename;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexSetDataFilename:invalidNumInputs",
            "Two inputs required.");
    
	filename = (char *) mxCalloc(mxGetN(prhs[1])+1, sizeof(char));
    mxGetString(prhs[1], filename, mxGetN(prhs[1])+1);      
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackSetDataFilename(filename);
}

/*Stop writing tracking data to the CSV file.*/
void mexCloseDataFile(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexCloseDataFile:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	
	y[0] = (int32_t) crsLiveTrackCloseDataFile();
}

/*This text will be written to a subsequent comment field
			entry in the CSV data file.*/
void mexSetDataComment(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	char *filename;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexSetDataComment:invalidNumInputs",
            "Two inputs required.");
    
	filename = (char *) mxCalloc(mxGetN(prhs[1])+1, sizeof(char));
    mxGetString(prhs[1], filename, mxGetN(prhs[1])+1);      
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackSetDataComment(filename);
}



/* ****************** The gateway function *********************  */

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	long functionIndex;

	if(nrhs < 1) {
		 mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nrhs",
								 "No option selected.");
	}

  functionIndex = (long) mxGetScalar(prhs[0]);

	switch (functionIndex) {
		case 0:
            mexInit(nlhs, plhs, nrhs, prhs);
            /*This function initialises LiveTrack. This function returns:
            negtive error code if unsuccesful
            1 for LiveTrack FM
            2 for LiveTrack AP
            3 for LiveTrack AV
            4 for LiveTrack Presto/Lightning */
			break;
		case 1:
            /*This function closes LiveTrack. This function returns 0 if
            successful. */
            mexClose(nlhs, plhs, nrhs, prhs);
			break;
		case 2:
            /*This function sets the type of eye tracking results to be 
            returned from LiveTrack. This function returns -100 if
            Livetrack is not initialized; -200 if the input parameter is
            not a recognised results type and should return 0 if
            successful. */
            mexSetResultsType(nlhs, plhs, nrhs, prhs);
			break;
        case 3:
            /*This function starts buffering (raw or calibrated) eye-
            tracking data and will continue to do so until 
            crsLiveTrackStopTracking is called. This function returns 0 
            if successful. */
			mexStartTracking(nlhs, plhs, nrhs, prhs);
			break;
		case 4:
            /*This function stops buffering (raw or calibrated)
            eye-tracking data. This function returns 0 if successful. */
			mexStopTracking(nlhs, plhs, nrhs, prhs);
			break;
        case 5:
            /*This function returns the number of results currently stored
            in the crsLiveTrack results buffer. */
			mexGetResultsCount(nlhs, plhs, nrhs, prhs);
			break;
        case 6:
            /*This function returns the results from the buffer. The 
            function returns 0 if successful or returns -1 if no result
            is returned. */
			mexGetResults(nlhs, plhs, nrhs, prhs);
			break;
		case 7:
            /*This function clears all results from the crsLiveTrack data
            buffer. This function returns 0 if successful. */
			mexClearDataBuffer(nlhs, plhs, nrhs, prhs);
			break;
        case 8:
            /*This function returns the version number of the Livetrack 
            library.  */
			mexGetLibVersion(nlhs, plhs, nrhs, prhs);
			break;
		case 9:
            /*This function calibrates the device for one eye using
            crsLiveTrack. This function will return 0 if successful or
            return -100 if LiveTrack has not been initialized (using the
            crsLiveTrackInit command). */
			mexCalibrateDevice(nlhs, plhs, nrhs, prhs);	
			break;
        case 10:
            /*This function sets the calibration matrix for either the left
            or right eye.     */
            mexSetCalibration(nlhs, plhs, nrhs, prhs);
            break;
        case 11:    
            /*This function returns the calibration matrix and rpc value
            for either the left or right eye. */
			mexGetCalibration(nlhs, plhs, nrhs, prhs);	
			break;
		case 12:
            /*This function sets the video image pixels-to-millimetre
            scaling factor. This set to a default value but can be
            modified by tracking a pupil or calibration dot target of 
            known diameter in the video stream. */
            mexSetPupilCalibration(nlhs, plhs, nrhs, prhs);	
			break;
        case 13:
            /*This function returns the video image pixels-to-millimetre
            scaling factor. This is set to a default value but can be
            modified by using crsLiveTrackSetPupilCalibration.*/
            mexGetPupilCalibration(nlhs, plhs, nrhs, prhs);	
			break;
        case 14:
            /*This function defines which eyes are to be tracked.*/
            mexSetTracking(nlhs, plhs, nrhs, prhs);	
            break; 
        case 15:
            /*This function returns which eyes are set to be tracked.*/
            mexGetTracking(nlhs, plhs, nrhs, prhs);	
            break;
        case 16:
            /*This function obtains the LiveTrack serial number.*/
            mexGetSerialNumber(nlhs, plhs, nrhs, prhs);	
            break;
        case 17:
            /*This function obtains the LiveTrack firmware version number.*/
            mexGetFirmwareVersion(nlhs, plhs, nrhs, prhs);	
            break;  
        case 18:
            /*This function obtains the LiveTrack build date and time.*/
            mexGetFirmwareBuildDateTime(nlhs, plhs, nrhs, prhs);	
            break;
        case 19:
            /*This function saves the gaze and pupil calibration to a file */
            mexSaveCalibration(nlhs, plhs, nrhs, prhs);	
            break;
        case 20:
            /*This function loads the gaze and pupil calibration from a file*/
            mexLoadCalibration(nlhs, plhs, nrhs, prhs);	
            break;
        case 21:
            /*Returns the device calibration state.
            Returns 0 on success. */
            mexGetCalibrated(nlhs, plhs, nrhs, prhs);
            break;
        case 22:
            /*Returns the device calibration state of the pupil.
            Returns 0 on success. */
            mexGetPupilCalibrated(nlhs, plhs, nrhs, prhs);
            break; 
        case 23:
            /*Returns true if tracking data is being saved into the buffer.*/
            mexGetTrackingStarted(nlhs, plhs, nrhs, prhs);
            break;
        case 24:
            /*Returns true if tracking data is being saved into the buffer.*/
            mexGetCaptureConfig(nlhs, plhs, nrhs, prhs);
            break;   
/*             MOVED TO crsLiveTrackGS.c
             case 25:
             //This function starts recording eyetracking video to file
             mexVideoStartRecord(nlhs, plhs, nrhs, prhs);
             break;   
 		  case 26:
             //This function stops recording eyetracking video to file
             mexVideoStopRecord(nlhs, plhs, nrhs, prhs);
             break;  */
		case 39:
			/*This function starts % recording eyetracking video to file.*/
			mexSetDataFilename(nlhs, plhs, nrhs, prhs);
			break;
		case 40:
			/*Stop writing tracking data to the CSV file.*/
			mexCloseDataFile(nlhs, plhs, nrhs, prhs);
			break;
		case 41:
			/*This text will be written to a subsequent comment field
			entry in the CSV data file.*/
			mexSetDataComment(nlhs, plhs, nrhs, prhs);
			break;
			
		default:
			break;
	}
}
