#include "mex.h"
#include "stdint.h"
#include "libLiveTrackGS.h"

#define VET_ASSERT(TEST,ERRORMESSAGE) ( (TEST) ? (void)0 : mexErrMsgTxt(ERRORMESSAGE) )

// ****************************************************************************
#define GET_pOUT(pDOUBLE_OUT,pMXARRAY_OUT,nROWS,nCOLS)  VET_ASSERT(0<nROWS,"Error in crsLiveTrack: Cannot create a matrix smaller than 1-by-1 in size"); \
                                                        VET_ASSERT(0<nCOLS,"Error in crsLiveTrack: Cannot create a matrix smaller than 1-by-1 in size"); \
                                                        pMXARRAY_OUT = mxCreateDoubleMatrix(nROWS,nCOLS,mxREAL);                                   \
                                                        VET_ASSERT(pMXARRAY_OUT!=NULL,"Out of memory - not enough free heap space");               \
                                                        pDOUBLE_OUT = (double *)mxGetPr(pMXARRAY_OUT);                                             \
                                                        VET_ASSERT(pDOUBLE_OUT!=NULL,"No real data in mxarray")

// ****************************************************************************

// written by Mark Hodgetts and Jakob Thomassen
// usage:
// mex crsLiveTrackGS.c libLiveTrackGS.lib                                                      
                                                    

void mexVideoStartRecord(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{    
    int32_t *y;
	char *filename;
	
	if(nrhs!=2) 
    mexErrMsgIdAndTxt( "MATLAB:mexVideoStartRecord:invalidNumInputs",
            "Two inputs required.");
    
	filename = (char *) mxCalloc(mxGetN(prhs[1])+1, sizeof(char));
    mxGetString(prhs[1], filename, mxGetN(prhs[1])+1);      
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
    crsLiveTrackVideoInit(0,false);
    
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t) crsLiveTrackVideoStartRecord(filename);
}

void mexVideoStopRecord(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	int32_t *y;
	
	if(nrhs!=1) 
	mexErrMsgIdAndTxt( "MATLAB:mexVideoStopRecord:invalidNumInputs",
            "One input required.");
	
	plhs[0] = mxCreateNumericMatrix(1, 1, mxINT32_CLASS, mxREAL);
  
	/* Assign pointer to the output */
	y = mxGetPr(plhs[0]);
	y[0] = (int32_t)crsLiveTrackVideoStopRecord();
}

// % EXPORT int32_t crsLiveTrackGetCaptureConfig(uint32_t *width, uint32_t *height, uint32_t *rate, uint32_t *offsetX, uint32_t *offsetY);

// ****************** The gateway function ********************* 

void mexFunction(int nlhs, mxArray *plhs[],
                 int nrhs, const mxArray *prhs[])
{
	long functionIndex;

	if(nrhs < 1) {
		 mexErrMsgIdAndTxt("MyToolbox:arrayProduct:nrhs",
								 "No option selected.");
	}

  functionIndex = (long) mxGetScalar(prhs[0]);

	switch (functionIndex) {
 
		  case 25:
            //This function starts recording eyetracking video to file
            mexVideoStartRecord(nlhs, plhs, nrhs, prhs);
            break;   
		  case 26:
            //This function stops recording eyetracking video to file
            mexVideoStopRecord(nlhs, plhs, nrhs, prhs);
            break;  
		default:
			break;
	}
}
