/*
Written by Mark Hodgetts & Shaun Simpson 11-07-2017 Copyright Cambridge Research Systems Ltd.

libLiveTrack library specific definitions.
*/
#ifndef LIBLIVETRACKGS_H__
#define LIBLIVETRACKGS_H__

#include <stdint.h>

enum LIVETRACKGS_ERROR {
	LIVETRACKGS_ERROR_OK = 0,
	LIVETRACKGS_ERROR_GSTREAMER_BUS = -11,
	LIVETRACKGS_ERROR_GSTREAMER_INIT = -12,
	LIVETRACKGS_ERROR_GSTREAMER_LINK = -13,
	LIVETRACKGS_ERROR_GSTREAMER_START = -14,
	LIVETRACKGS_ERROR_UNKNOWN = -1000,
};

#ifdef __cplusplus
extern "C" {
#endif
	uint32_t crsLiveTrackGetLibGSVersion(void);


	/* crsLiveTrackVideoInit initialises the GStreamer video layer.

	Paramaters:
	windowHandle - The window handle parameter can be used to display the video in a widget e.g. A panel.
	               A value of NULL will open a seperate wiindow for video display.

	directx - (Windows only) true = use directx, false = use OpenGL for video display.
	deviceindex - (macOS only) used to select the index of the video capture device.

	Return:
	LIVETRACKGS_ERROR_OK - on success.
	*/
#ifdef _WIN32
	int32_t crsLiveTrackVideoInit(uintptr_t windowHandle, bool directx);
#else
	int32_t crsLiveTrackVideoInit(uintptr_t windowHandle, int deviceIndex);
#endif

	/* crsLiveTrackVideoStart start video playback.
	Return:
	LIVETRACKGS_ERROR_OK - on success.
	*/
	int32_t crsLiveTrackVideoStart(void);

	/* crsLiveTrackVideoFree stops video playback.
	Return:
	LIVETRACKGS_ERROR_OK - on success.
	*/
	int32_t crsLiveTrackVideoStop(void);

	/* crsLiveTrackVideoStartRecord start video capture recording.

	Paramaters:
	filename - Filename to save video file (unicode not supported).

	Return:
	LIVETRACKGS_ERROR_OK - on success.
	*/
	int32_t crsLiveTrackVideoStartRecord(char *filename);

	/* crsLiveTrackVideoStopRecord stops recording and restarts video playback if it was running before recordng.
	Return:
	LIVETRACKGS_ERROR_OK - on success.
	*/
	int32_t crsLiveTrackVideoStopRecord(void);

	/* crsLiveTrackVideoStopped returns true if the video is NOT playing. */
	int32_t crsLiveTrackVideoStopped(void);
#ifdef __cplusplus
}
#endif

#endif
