function [ deviceType ] = crsLiveTrackInit
% CRSLIVETRACKINIT initialises the crsLiveTrack MATLAB Toolbox
% -------------------------------------------------------------------------
% This function initialises the crsLiveTrack MATLAB Toolbox
% and establishes a connection to a CRS LiveTrack device. 
% No input parameters required.
%
% MATLAB Syntax
% =============
% [ deviceType ] = crsLiveTrackInit
%
% Return values
% =============
% deviceType - An integer (1-by-1 matrix) with one of the following values:
%
% -100  - ERROR: LiveTrack device not found!
% 1     - LiveTrack FM initialised.
% 2     - LiveTrack AP initialised.
% 3     - LiveTrack AV initialised.
% 4     - LiveTrack Presto initialised.
%
% History:
% 09/2017   MAH Created  
%
%% Load the Toolbox Functions and Initialise

% Load Toolbox Constants
crsLiveTrackLoadConstants;

% Call initialise function and get device type
deviceType = crsLiveTrack(CRS.mexInit);

if (deviceType<1)
    error('ERROR: LiveTrack device not found! Is the data interface (HID) still connected to the LiveTrack Viewer App? Close the app or detach the data interface');
end
if (deviceType==1)
    disp('LiveTrack FM initialised.');
elseif (deviceType==2)
    disp('LiveTrack AP initialised.');
elseif (deviceType==3)
    disp('LiveTrack AV initialised.');
elseif (deviceType==4)
    disp('LiveTrack Presto initialised.');
end

