function [ err ] = crsLiveTrackSetPupilCalibration(diameter, pixels)
% CRSLIVETRACKSETPUPILCALIBRATION This function sets the pupil size scale
% factor of CRS LiveTrack. It is meant to be used after raw data has been
% obtained while tracking an artificial pupil of a known diameter (e.g the
% supplied Pupil Scale Stick)
% -------------------------------------------------------------------------
%
% MATLAB Syntax
% =============
% [ err ] = crsLiveTrackSetPupilCalibration(diameter, pixels)
%
% diameter - the size (in millimetres) of the artificial pupil 
%    
% pixels - the major axis of the tracked artificial pupil (e.g. R.LPupilMa)
%
% Return values
% =============
% err - This will be zero if pupil calibration is successfully set
%
% History:
% 07/2017   JT Created  

%% Access the Toolbox Constants and get the pupil size calibration

% Access Toolbox Constants
global CRS;

   
% Get the pupil size calibration
err = crsLiveTrack(CRS.mexSetPupilCalibration, diameter, pixels);

if (err~=0)
    disp('ERROR: LiveTrack pupil size scale factor not set!');
end

