function [ err ] = crsLiveTrackSetResultsTypeRaw
% CRSLIVETRACKSETRESULTSTYPERAW This function tells LiveTrack to send raw
% data rather than calibrated data.
% No input parameters required.
%
% Raw data consists of the x and y coordinates of the centre location of 
% the pupil and glint in the camera image. In dual glint systems the
% geometric average will be returned. The units will be retuned in pixels
% of the camera sensor. 
% The pupil ellipse fit returns the major and minor axes in camera pixels.
% -------------------------------------------------------------------------
%
% MATLAB Syntax
% =============
% [ err ] = crsLiveTrackSetResultsTypeRaw
%
% Return values
% =============
% err - An integer where 0 indicates the results type has been successfully
% set to raw
%
%
% History:
% 07/2017   JT Created  

%%

% Load toolbox constants
global CRS;

% Call to set the results type of raw type 
err = crsLiveTrack(CRS.mexSetResultsType, CRS.raw);

if err==0
    disp('Results type set to Raw.');
else
    error('ERROR: Could not set results type.');
end
