function [ err ] = crsLiveTrackSetTracking( leftEye, rightEye )
% CRSLIVETRACKSETTRACKING This function sets the tracking states for the left and
% right eyes on CRS LiveTrack. See also crsLiveTrackGetTracking
% -------------------------------------------------------------------------
%
% MATLAB Syntax
% =============
% [ err ] = crsLiveTrackSetTracking( leftEye, rightEye )
%
% leftEye - set to true (or false) if left eye is to be tracked (not tracked) 
%
% rightEye - set to true (or false) if right eye is to be tracked (not tracked) 
%
% Return values
% =============
% err - An integer where 0 indicates eye tracking has been successfully set 
%
%
% History:
% 09/2017   JT Created  
%
%% Access the Toolbox Constants and Set the tracking state

% Access Toolbox Constants
global CRS;

% Set the tracking state
[ err ] = crsLiveTrack(CRS.mexSetTracking, leftEye, rightEye);

if (err==0)
    if leftEye && rightEye
        disp('LIVETRACK: tracking set to use both eyes');
    elseif leftEye
        disp('LIVETRACK: tracking set to use left eye only');
    elseif rightEye
        disp('LIVETRACK: tracking set to use right eye only');
    end
end
if (err~=0)
    disp('ERROR: LiveTrack tracking state not set!');
end
