function [ err ] = crsLiveTrackStartVideoRecord(filename)
% CRSLIVETRACKSTARTVIDEORECORD This function starts 
% recording eyetracking video to file.
% -------------------------------------------------------------------------
% NB. Make sure you have GStreamer version 1.12.4 or later
%
% MATLAB Syntax
% =============
% [ err ] = crsLiveTrackStartVideoRecord(filename)
%
% filename - the filename to record to including the path
%
% Return values
% =============
% err - An integer where 0 indicates video recording has successfully started 
%
% Example
% =======
% crsLiveTrackStartVideoRecord('C:\tmp\recorded_video.avi')
%
% History:
% 09/2017   MAH Created  
% 01/2018   JT Updated to separate mex for GStreamer
%
%% Access the Toolbox Constants and Start Recording

% Access Toolbox Constants
global CRS;

% Start Recording
if IsWin
    newStr = strrep(filename,'\','\\'); % replace single backslash with double backslash
    err = crsLiveTrackGS(CRS.mexVideoStartRecord, newStr);
elseif IsOSX
    newStr = filename;
    err = crsLiveTrackGS(CRS.mexVideoStartRecord, newStr);
end

if (err~=0)
    disp('ERROR: LiveTrack video recording not started!');
end
if (err==-200)
    disp('ERROR: GStreamer version unsupported. Version 1.12.4 or later needed');
end
if (err==0)
    disp('LiveTrack video recording started.');
end
