function crsLiveTrackDemoPupillometry(WindowID)
%CRSLIVETRACKDEMOPUPILLOMETRY  This simple demo shows how to use the CRS
%   LiveTrack eye tracker to measure pupil size
%
%   The demo uses Psychtoolbox to alternate between a full white screen and
%   a full black screen. During the stimulation the pupil size will be
%   continously measured and finally the measurement will be ploted
%
%   The input to this function must be:
%   WindowID - the ID of the stimulus monitor as specified by the function
%   Screen('Screens'). If no input is given, the monitor with the highest 
%   ID will be used (i.e. the secondary monitor if present).

% History:
% 06/2017 JT Created


framesPerFlash = 120;

% Define the raduis of the fixation point (in screen pixels) 
fixRadIn = 4; % inner circle
fixRadOut = 20;  % outer circle

%% Default value for input parameter (if omitted)

% Define default value for the monitor to use 
if nargin<1
    % Get the IDs of all monitors present
    screensAvaiable = Screen('Screens');

   % ID of the stimulus monitor
    WindowID(1) = max(screensAvaiable);
end


%% Setup LiveTrack

% Initialise LiveTrack
crsLiveTrackInit;

% Start streaming calibrated results
crsLiveTrackSetResultsTypeCalibrated;

% Start buffering data to the library
crsLiveTrackStartTracking;

% Use default settings for Psychtoolbox
PsychDefaultSetup(2);

% Open a full screen window with grey background on stimulus monitor
winStim = PsychImaging('OpenWindow', WindowID, 0.5);

% get the resolution of the stimulus monitor
ResStim=Screen('Resolution', WindowID);

% Loop for 10 seconds
c=0;
tic;
while toc<10
    c=c+1;
    
    % Draw
    if c>framesPerFlash
        Screen('FillRect', winStim, 1);
        if c>framesPerFlash*2
            c=0;
        end
    else
        Screen('FillRect', winStim, 0);
    end
    Screen('FillOval', winStim, 255, round([ResStim.width/2-fixRadOut...
    ResStim.height/2-fixRadOut ResStim.width/2+fixRadOut ResStim.height/2+fixRadOut]));
    Screen('FillOval', winStim, 0, round([ResStim.width/2-fixRadIn...
    ResStim.height/2-fixRadIn ResStim.width/2+fixRadIn ResStim.height/2+fixRadIn]));

    % Update the stimulus screen
    Screen('Flip', winStim);
end

% Close all PTB windows
sca;

% Stop buffering data to the library
crsLiveTrackStopTracking;

Data = crsLiveTrackGetBufferedEyePositions;

% Clear the buffer
crsLiveTrackClearDataBuffer;

% Close LiveTrack
crsLiveTrackClose;

figure;
plot(Data.timeStamps./1000,Data.pupilDiameter,'r'); hold on;
plot(Data.timeStamps./1000,Data.pupilDiameterRight,'g'); hold on;
xlabel('time (mS)');
ylabel('Pupil Diameter (mm)');