% function crsLiveTrackDemoROIsimple
% CRSLIVETRACKDEMOROISIMPLE   This demo shows how to check if a fixation
% was held during a trail using a Cambridge Research Systems LiveTrack eye
% tracker
%
% NB. The Virtual pins no. 0 and 1 must be set in the config.xml file. E.g.:
% 
%       <Virtual0>
% 			<Function>LeftROI</Function>
% 			<TopLeftX>-20</TopLeftX>
% 			<TopLeftY>-20</TopLeftY>
% 			<BottomRightX>20</BottomRightX>
% 			<BottomRightY>20</BottomRightY>
% 			<Default>0</Default>
% 		</Virtual0>
% 		
% 		<Virtual1>
% 			<Function>RightROI</Function>
% 			<TopLeftX>-20</TopLeftX>
% 			<TopLeftY>-20</TopLeftY>
% 			<BottomRightX>20</BottomRightX>
% 			<BottomRightY>20</BottomRightY>
% 			<Default>0</Default>
% 		</Virtual1>
%
%  This will set the Virtual pin no. 0 and 1 to be high when the left and
% right eye (repectively) is within window with a side length of 40 mm 
% centred on the screen
%
% The demo uses the CRS LiveTrack Toolbox for MATLAB

% History:
% 11/2017 JT Created

% Initialise LiveTrack
crsLiveTrackInit;

% Set LiveTrack to return calibrated data
crsLiveTrackSetResultsTypeCalibrated;

% Clear the buffer
crsLiveTrackClearDataBuffer;

% Start Tracking
crsLiveTrackStartTracking;

% Do trial here
pause(10); % paused for 10 seconds to simulate trial

% Stop Tracking
crsLiveTrackStopTracking;

% Get all the data in the buffer
Data = crsLiveTrackGetBufferedEyePositions;

leftROI = crsLiveTrackGetDigitalIOpin(Data,'Virtual0'); % this is left eye ROI
rightROI = crsLiveTrackGetDigitalIOpin(Data,'Virtual1'); % this is left eye ROI

disp(['The left eye was in the ROI for ',num2str(100*sum(leftROI)/length(leftROI)),' % of the time'])
disp(['The right eye was in the ROI for ',num2str(100*sum(rightROI)/length(rightROI)),' % of the time'])

% Clear the buffer
crsLiveTrackClearDataBuffer;

% Close LiveTrack
crsLiveTrackClose;


