% function crsLiveTrackDemoWriteCalibrationToConfigFile
% CRSLIVETRACKDEMOWRITECALIBRATIONTOCONFIGFILE   This demo shows how to get
% the calibration from CRS LiveTrack and write it to a file (called 
% calibration.txt written to the MATLAB Current Folder). This can then 
% be used to set a default calibration that will be loaded every time the
% LiveTrack is turned on by replacing the data in the Config.xml file. 
%
% The demo uses the CRS LiveTrack Toolbox for MATLAB

% History:
% 04/2018 JT Created

filename = 'calibration.txt';

% Initialise LiveTrack
crsLiveTrackInit;

[ calibrationL, viewDistL ] = crsLiveTrackGetCalibration('leftEye');
[ calibrationR, viewDistR ] = crsLiveTrackGetCalibration('rightEye');

if viewDistL~=viewDistR
    error('viewing distance in left and rught eye does not match');
end

% Close LiveTrack
crsLiveTrackClose;


%% generate the file

fid = fopen(filename, 'w+');


matEntry=[11,12,13,14,21,22,23,24,31,32,33,34,41,42,43,44];

% Left eye
for i=1:16 
    fprintf(fid,'\t<Entry Left_eye_A%d="%f" />\n',matEntry(i),calibrationL(i));
end
fprintf(fid,'\t<Entry Left_eye_RPC="100" />\n');

% Right eye
for i=1:16 
    fprintf(fid,'\t<Entry Right_eye_A%d="%f" />\n',matEntry(i),calibrationR(i));
end
fprintf(fid,'\t<Entry Right_eye_RPC="100" />\n');

fclose(fid);
disp('Calibration data written to:')
disp([cd,filesep,filename]);